@@ -1,4 +1,4 @@
-# $Id: HTTPD.pm 2272 2008-02-20 10:18:36Z bingosnet $
+# $Id: HTTPD.pm,v 1.36 2004/11/16 07:12:44 rcaputo Exp $
 
 # Filter::HTTPD Copyright 1998 Artur Bergman <artur@vogon.se>.
 
@@ -9,29 +9,22 @@
 # 2001-07-27 RCC: This filter will not support the newer get_one()
 # interface.  It gets single things by default, and it does not
 # support filter switching.  If someone absolutely needs to switch to
-# and from HTTPD filters, they should submit their request as a patch.
+# and from HTTPD filters, they should say so on POE's mailing list.
 
 package POE::Filter::HTTPD;
+use POE::Preprocessor ( isa => "POE::Macro::UseBytes" );
 
 use strict;
-use POE::Filter;
 
-use vars qw($VERSION @ISA);
-$VERSION = do {my($r)=(q$Revision: 2272 $=~/(\d+)/);sprintf"1.%04d",$r};
-@ISA = qw(POE::Filter);
-
-sub BUFFER        () { 0 }
-sub TYPE          () { 1 }
-sub FINISH        () { 2 }
-sub HEADER        () { 3 }
-sub CLIENT_PROTO  () { 4 }
+use vars qw($VERSION);
+$VERSION = do {my@r=(q$Revision: 1.36 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Carp qw(croak);
-use HTTP::Status qw( status_message RC_BAD_REQUEST RC_OK RC_LENGTH_REQUIRED );
-use HTTP::Request ();
-use HTTP::Response ();
+use HTTP::Status;
+use HTTP::Request;
+use HTTP::Response;
 use HTTP::Date qw(time2str);
-use URI ();
+use URI;
 
 my $HTTP_1_0 = _http_version("HTTP/1.0");
 my $HTTP_1_1 = _http_version("HTTP/1.1");
@@ -40,46 +33,28 @@ my $HTTP_1_1 = _http_version("HTTP/1.1");
 
 sub new {
   my $type = shift;
-  my $self = [
-    '',     # BUFFER
-    0,      # TYPE
-    0,      # FINISH
-    undef,  # HEADER
-    undef,  # CLIENT_PROTO
-  ];
+  my $self = { type   => 0,
+               buffer => '',
+               finish => 0,
+             };
   bless $self, $type;
   $self;
 }
 
 #------------------------------------------------------------------------------
 
-sub get_one_start {
-    my ($self, $stream) = @_;
-    return if ( $self->[FINISH] );
-    $stream = [ $stream ] unless ( ref( $stream ) );
-    $self->[BUFFER] .= join( '', @$stream );
-}
-
-sub get_one {
-    my ($self) = @_;
-    return ( $self->[FINISH] ) ? [] : $self->get( [] );
-}
-
 sub get {
   my ($self, $stream) = @_;
 
-  # Need to check lengths in octets, not characters.
-  BEGIN { eval { require bytes } and bytes->import; }
+  {% use_bytes %}
 
-  # Why?
   local($_);
 
   # Sanity check.  "finish" is set when a request has completely
   # arrived.  Subsequent get() calls on the same request should not
-  # happen.
-  # TODO Maybe this should return [] instead of dying?
+  # happen.  -><- Maybe this should return [] instead of dying?
 
-  if ($self->[FINISH]) {
+  if($self->{finish}) {
 
     # This works around a request length vs. actual content length
     # error.  Looks like some browsers (mozilla!) sometimes add on an
@@ -89,7 +64,7 @@ sub get {
 
     my @dump;
     my $offset = 0;
-    $stream = $self->[BUFFER].join("", @$stream);
+    $stream = join("", @$stream);
     while (length $stream) {
       my $line = substr($stream, 0, 16);
       substr($stream, 0, 16) = '';
@@ -102,172 +77,122 @@ sub get {
       $offset += 16;
     }
 
-    return [
-      $self->_build_error(
-        RC_BAD_REQUEST,
-        "Did not want any more data.  Got this:" .
-        "<p><pre>" . join("", @dump) . "</pre></p>"
-      )
-    ];
+    return [ $self->build_error
+             ( RC_BAD_REQUEST,
+               "Did not want any more data.  Got this:" .
+               "<p><pre>" . join("", @dump) . "</pre></p>"
+             )
+           ];
   }
 
   # Accumulate data in a framing buffer.
 
-  $self->[BUFFER] .= join('', @$stream);
+  $self->{buffer} .= join('', @$stream);
 
   # If headers were already received, then the framing buffer is
   # purely content.  Return nothing until content-length bytes are in
   # the buffer, then return the entire request.
 
-  if ($self->[HEADER]) {
-    my $buf = $self->[BUFFER];
-    my $r   = $self->[HEADER];
-    my $cl  = $r->content_length() || length($buf) || 0;
-
-    # Some browsers (like MSIE 5.01) send extra CRLFs after the
-    # content.  Shame on them.  Now we need a special case to drop
-    # their extra crap.
-    #
-    # We use the first $cl octets of the buffer as the request
-    # content.  It's then stripped away.  Leading whitespace in
-    # whatever is left is also stripped away.  Any nonspace data left
-    # over will throw an error.
-    #
-    # Four-argument substr() would be ideal here, but it's a
-    # relatively recent development.
-    #
-    # PG- CGI.pm only reads Content-Length: bytes from STDIN.
+  if($self->{header}) {
+    my $buf = $self->{buffer};
+    my $r   = $self->{header};
+    my $cl  = $r->content_length() || "0 (implicit)";
     if (length($buf) >= $cl) {
-      $r->content(substr($buf, 0, $cl));
-      $self->[BUFFER] = substr($buf, $cl);
-      $self->[BUFFER] =~ s/^\s+//;
-
-      # We are sending this back, so won't need it anymore.
-      $self->[HEADER] = undef;
-      $self->[FINISH]++;
+      $r->content($buf);
+      $self->{finish}++;
       return [$r];
+    } else {
+      # print "$cl wanted, got " . length($buf) . "\n";
     }
-
-    #print "$cl wanted, got " . length($buf) . "\n";
     return [];
   }
 
   # Headers aren't already received.  Short-circuit header parsing:
   # don't return anything until we've received a blank line.
 
-  return [] unless(
-    $self->[BUFFER] =~ /(\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?)/s
-  );
+  return []
+    unless($self->{buffer} =~/(\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?)/s);
 
   # Copy the buffer for header parsing, and remove the header block
   # from the content buffer.
 
-  my $buf = $self->[BUFFER];
-  $self->[BUFFER] =~ s/.*?(\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?)//s;
+  my $buf = $self->{buffer};
+  $self->{buffer} =~s/.*?(\x0D\x0A?\x0D\x0A?|\x0A\x0D?\x0A\x0D?)//s;
 
   # Parse the request line.
+
   if ($buf !~ s/^(\w+)[ \t]+(\S+)(?:[ \t]+(HTTP\/\d+\.\d+))?[^\012]*\012//) {
-    return [
-      $self->_build_error(RC_BAD_REQUEST, "Request line parse failure.")
-    ];
+    return [ $self->build_error(RC_BAD_REQUEST, "Request line parse failure.") ];
   }
   my $proto = $3 || "HTTP/0.9";
 
   # Use the request line to create a request object.
 
-  my $method = $1;
-  my $req_path = $2;
-  $req_path =~ s/^[\/]{2,}/\//; # fix double slash starting path
-
-  my $r = HTTP::Request->new($method, URI->new($req_path));
+  my $r = HTTP::Request->new($1, URI->new($2));
   $r->protocol($proto);
-  $self->[CLIENT_PROTO] = $proto = _http_version($proto);
+  $self->{'httpd_client_proto'} = $proto = _http_version($proto);
 
   # Add the raw request's headers to the request object we'll be
   # returning.
 
-  if ($proto >= $HTTP_1_0) {
+  if($proto >= $HTTP_1_0) {
     my ($key,$val);
-    HEADER: while ($buf =~ s/^([^\012]*)\012//) {
+  HEADER:
+    while ($buf =~ s/^([^\012]*)\012//) {
       $_ = $1;
       s/\015$//;
       if (/^([\w\-~]+)\s*:\s*(.*)/) {
         $r->push_header($key, $val) if $key;
         ($key, $val) = ($1, $2);
-      }
-      elsif (/^\s+(.*)/) {
+      } elsif (/^\s+(.*)/) {
         $val .= " $1";
-      }
-      else {
+      } else {
         last HEADER;
       }
     }
     $r->push_header($key,$val) if($key);
   }
 
-  $self->[HEADER] = $r;
+  $self->{header} = $r;
 
   # If this is a GET or HEAD request, we won't be expecting a message
   # body.  Finish up.
-  $method = uc $r->method();
+
+  my $method = $r->method();
   if ($method eq 'GET' or $method eq 'HEAD') {
-    $self->[FINISH]++;
-    # We are sending this back, so won't need it anymore.
-    $self->[HEADER] = undef;
+    $self->{finish}++;
     return [$r];
   }
 
-  # However, if it's any other type of request, check whether the
-  # entire content has already been received!  If so, add that to the
-  # request and we're done.  Otherwise we'll expect a subsequent get()
-  # call to finish things up.
+  # However, if it's a POST request, check whether the entire content
+  # has already been received!  If so, add that to the request and
+  # we're done.  Otherwise we'll expect a subsequent get() call to
+  # finish things up.
 
-  #print "post:$buf:\END BUFFER\n";
-  #print length($buf)."-".$r->content_length()."\n";
+  if($method eq 'POST') {
 
-  my $cl = $r->content_length();
-  unless(defined $cl) {
-    if($self->[CLIENT_PROTO] == 9) {
-      return [
-        $self->_build_error(
-          RC_BAD_REQUEST,
-          "POST request detected in an HTTP 0.9 transaction. " .
-          "POST is not a valid HTTP 0.9 transaction type. " .
-          "Please verify your HTTP version and transaction content."
-        )
-      ];
+#    print "post:$buf:\END BUFFER\n";
+#    print length($buf)."-".$r->content_length()."\n";
+
+    my $cl = $r->content_length();
+    unless(defined $cl) {
+        if($self->{'httpd_client_proto'} == 9) {
+            return [ $self->build_error(RC_BAD_REQUEST,  "POST request detected in an HTTP 0.9 transaction. POST is not a valid HTTP 0.9 transaction type. Please verify your HTTP version and transaction content.") ];
+
+        } else { 
+            return [ $self->build_error(RC_LENGTH_REQUIRED, 
+                        "No content length found.") ];
+        }
     }
-    elsif ($method eq 'OPTIONS') {
-      $self->[FINISH]++;
-      # OPTIONS requests can have an optional content length
-      # See http://www.faqs.org/rfcs/rfc2616.html, section 9.2
-      $self->[HEADER] = undef;
+        
+    return [ $self->build_error(RC_BAD_REQUEST, "Content length contains non-digits.") ] 
+        unless $cl =~ /^\d+$/;
+
+    if (length($buf) >= $cl) {
+      $r->content($buf);
+      $self->{finish}++;
       return [$r];
     }
-    else {
-      return [
-        $self->_build_error(RC_LENGTH_REQUIRED, "No content length found.")
-      ];
-    }
-  }
-
-  unless ($cl =~ /^\d+$/) {
-    return [
-      $self->_build_error(
-        RC_BAD_REQUEST,
-        "Content length contains non-digits."
-      )
-    ];
-  }
-
-  if (length($buf) >= $cl) {
-    $r->content(substr($buf, 0, $cl));
-    $self->[BUFFER] = substr($buf, $cl);
-    $self->[BUFFER] =~ s/^\s+//;
-    $self->[FINISH]++;
-    # We are sending this back, so won't need it anymore.
-    $self->[HEADER] = undef;
-    return [$r];
   }
 
   return [];
@@ -293,7 +218,7 @@ sub put {
 
     my $status_line = "$proto $code";
     $status_line   .= " ($status_message)"  if $status_message ne $message;
-    $status_line   .= " $message" if length($message);
+    $status_line   .= " $message";
 
     # Use network newlines, and be sure not to mangle newlines in the
     # response's content.
@@ -305,9 +230,6 @@ sub put {
     push @raw, join("\x0D\x0A", @headers, "") . $_->content;
   }
 
-  # Allow next request after we're done sending the response.
-  $self->[FINISH]--;
-
   \@raw;
 }
 
@@ -320,7 +242,7 @@ sub get_pending {
 }
 
 #------------------------------------------------------------------------------
-# Functions specific to HTTPD;
+# function specific to HTTPD;
 #------------------------------------------------------------------------------
 
 # Internal function to parse an HTTP status line and return the HTTP
@@ -335,11 +257,10 @@ sub _http_version {
 # Build a basic response, given a status, a content type, and some
 # content.
 
-sub _build_basic_response {
+sub build_basic_response {
   my ($self, $content, $content_type, $status) = @_;
 
-  # Need to check lengths in octets, not characters.
-  BEGIN { eval { require bytes } and bytes->import; }
+  {% use_bytes %}
 
   $content_type ||= 'text/html';
   $status       ||= RC_OK;
@@ -353,27 +274,28 @@ sub _build_basic_response {
   return $response;
 }
 
-sub _build_error {
+sub build_error {
   my($self, $status, $details) = @_;
 
   $status  ||= RC_BAD_REQUEST;
   $details ||= '';
   my $message = status_message($status) || "Unknown Error";
 
-  return $self->_build_basic_response(
-    ( "<html>" .
-      "<head>" .
-      "<title>Error $status: $message</title>" .
-      "</head>" .
-      "<body>" .
-      "<h1>Error $status: $message</h1>" .
-      "<p>$details</p>" .
-      "</body>" .
-      "</html>"
-    ),
-    "text/html",
-    $status
-  );
+  return
+    $self->build_basic_response
+      ( ( "<html>" .
+          "<head>" .
+          "<title>Error $status: $message</title>" .
+          "</head>" .
+          "<body>" .
+          "<h1>Error $status: $message</h1>" .
+          "<p>$details</p>" .
+          "</body>" .
+          "</html>"
+        ),
+        "text/html",
+        $status
+      );
 }
 
 ###############################################################################
@@ -435,20 +357,20 @@ example, the following code (taken almost verbatim from the
 HTTP::Request::Common documentation) will cause an error in a Filter::HTTPD
 daemon:
 
-  use HTTP::Request::Common;
-  use LWP::UserAgent;
+    use HTTP::Request::Common;
+    use LWP::UserAgent;
 
-  my $ua = LWP::UserAgent->new();
-  $ua->request(POST 'http://some/poe/driven/site', [ foo => 'bar' ]);
+    my $ua = LWP::UserAgent->new();
+    $ua->request(POST 'http://some/poe/driven/site', [ foo => 'bar' ]);
 
 By default, HTTP::Request is HTTP version agnostic. It makes no attempt to add
 an HTTP version header unless you specifically declare a protocol using
-C<< $request->protocol('HTTP/1.0') >>.
+C<< $request->protocol('HTTP/1.0') >>. 
 
 According to the HTTP 1.0 RFC (1945), when faced with no HTTP version header,
 the parser is to default to HTTP/0.9. Filter::HTTPD follows this convention. In
 the transaction detailed above, the Filter::HTTPD based daemon will return a 400
-error since POST is not a valid HTTP/0.9 request type.
+error since POST is not a valid HTTP/0.9 request type.  
 
 =head1 Streaming Media
 
@@ -485,6 +407,3 @@ The HTTPD filter was contributed by Artur Bergman.
 Please see L<POE> for more information about authors and contributors.
 
 =cut
-
-# rocco // vim: ts=2 sw=2 expandtab
-# TODO - Redocument.
