@@ -1,11 +1,11 @@
-# $Id: FollowTail.pm,v 1.49 2004/01/28 23:19:11 rcaputo Exp $
+# $Id: FollowTail.pm,v 1.51 2004/11/16 07:12:46 rcaputo Exp $
 
 package POE::Wheel::FollowTail;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my@r=(q$Revision: 1.49 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+$VERSION = do {my@r=(q$Revision: 1.51 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
 
 use Carp;
 use Symbol;
@@ -62,16 +62,6 @@ sub new {
 
   croak "$type requires a working Kernel" unless (defined $poe_kernel);
 
-  # STATE-EVENT
-  if (exists $params{InputState}) {
-    croak "InputState is deprecated.  Use InputEvent";
-  }
-
-  # STATE-EVENT
-  if (exists $params{ErrorState}) {
-    croak "ErrorState is deprecated.  Use ErrorEvent";
-  }
-
   croak "FollowTail requires a Handle or Filename parameter, but not both"
     unless $params{Handle} xor defined $params{Filename};
 
@@ -413,11 +403,6 @@ sub event {
   while (@_) {
     my ($name, $event) = splice(@_, 0, 2);
 
-    # STATE-EVENT
-    if ($name =~ /^(.*?)State$/) {
-      croak "$name is deprecated.  Use $1Event";
-    }
-
     if ($name eq 'InputEvent') {
       if (defined $event) {
         $self->[SELF_EVENT_INPUT] = $event;
@@ -589,7 +574,7 @@ time.  Please be careful.
 The SeekBack parameter tells FollowTail how far back from the end of
 the file to start reading.  Its value is specified in bytes, and
 values greater than the file's current size will quietly cause
-FollowTail to ltart from the file's beginning.
+FollowTail to start from the file's beginning.
 
 A SeekBack parameter of 0 starts FollowTail at the end of the file.
 It's recommended to omit Seek and SeekBack to start from the end of a
